//
// File: getHTTPFile.c
// Original Author: Jeff Huber
// Last Modified By: Jason Kane
// Last Modified: 2000-05-10
//
// Implements am_gethttpfile Tcl command. 
//
// Copyright (C) 1999 AM Computers
// 

// Link with NSD.LIB and TCL74.LIB for AOLserver 2.1 on NT

#include <ns.h>

#ifdef NS_AOLSERVER_3_PLUS
#include <tcl.h>
#else
#include <nsmem.h>
#include <nstcl.h>
#endif

DllExport int Ns_ModuleVersion = 1;

static Ns_TclInterpInitProc  HTTPFileInterpInit;
static Tcl_CmdProc am_getHTTPFileCmd;

// Makes module compile cleanly under AS 2.1 and 3.0
#define Ns_Malloc  ns_malloc
#define Ns_Free    ns_free
#define Ns_Realloc ns_realloc

DllExport 
int Ns_ModuleInit(char *hServer, char *hModule) {

	// Call AddGetHTTPFileCommand to create the Tcl command am_gethttpfile
	// in the parent interpreter.
	Ns_TclInitInterps(hServer, HTTPFileInterpInit, NULL);

	return NS_OK;
}

static int HTTPFileInterpInit (Tcl_Interp *interp, void *context) {

	// Create the am_gethttpfile command
	Tcl_CreateCommand(interp, "am_gethttpfile", am_getHTTPFileCmd, (ClientData) NULL,
		(Tcl_CmdDeleteProc *) NULL);

	return NS_OK;
}

static int am_getHTTPFileCmd (ClientData clientData, Tcl_Interp *interp, int argc, 
					char *argv[]) {
//
// Last Modified: 2000-01-05
// Last Modified By: Jason Kane
// Original Author: Jeff Huber
//
// Implements am_httpgetfile Tcl command. am_httpgetfile grabs a url
// and places it in a file. The command expects two arguments:
// the url and filename . We will overwrite the file if it already 
// exits.
//

	FILE *fp;
	Ns_DString contents;
		
	// Temp variables used in getting uri, host, and port from url
	const int httpProtoLength = 7; // length of string "http://"
	size_t firstSlash=0;
	size_t firstColon=0;

	// Make sure we were passed correct number of arguments
	if (argc != 3) {
		Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0], " url filename\"", NULL);
		return TCL_ERROR;
	}

	// Make sure url is http
	if (strncmp(argv[1], "http://", httpProtoLength)  != 0) {
		Tcl_AppendResult(interp, "Invalid URL: \"", argv[1], "\" We only support http", NULL);
		return TCL_ERROR;
	}

	Ns_DStringInit(&contents);
	Ns_FetchURL(&contents, argv[1], NULL);

	// Write data to file.
	if ((fp = fopen(argv[2], "wb")) == NULL) {
		Tcl_AppendResult(interp, "Error opening file \"", argv[2], "\" for writing", NULL);
		return TCL_ERROR;
	}

	if ((int)fwrite(Ns_DStringValue(&contents), sizeof(char), Ns_DStringLength(&contents), fp) < Ns_DStringLength(&contents)) {
		Tcl_AppendResult(interp, "Error writing data to file: \"", argv[2], "\"", NULL);
		fclose(fp);
		return TCL_ERROR;
	}

	// Cleanup.
	fclose(fp);
	return TCL_OK;
}

